@@ -4,14 +4,34 @@ import pytest
 
 from bleach import clean
 from bleach.html5lib_shim import Filter
-from bleach.sanitizer import Cleaner
+from bleach.sanitizer import ALLOWED_PROTOCOLS, Cleaner
 from bleach._vendor.html5lib.constants import rcdataElements
 
 
-def test_clean_idempotent():
+@pytest.mark.parametrize(
+    "data",
+    [
+        "a < b",
+        "link http://link.com",
+        "text<em>",
+        # Verify idempotentcy with character entity handling
+        "<span>text & </span>",
+        "jim &current joe",
+        "&&nbsp; &nbsp;&",
+        "jim &xx; joe",
+        # Link with querystring items
+        '<a href="http://example.com?foo=bar&bar=foo&amp;biz=bash">',
+    ],
+)
+def test_clean_idempotent(data):
     """Make sure that applying the filter twice doesn't change anything."""
-    dirty = "<span>invalid & </span> < extra http://link.com<em>"
-    assert clean(clean(dirty)) == clean(dirty)
+    assert clean(clean(data)) == clean(data)
+
+
+def test_clean_idempotent_img():
+    tags = ["img"]
+    dirty = '<imr src="http://example.com?foo=bar&bar=foo&amp;biz=bash">'
+    assert clean(clean(dirty, tags=tags), tags=tags) == clean(dirty, tags=tags)
 
 
 def test_only_text_is_cleaned():
@@ -56,10 +76,6 @@ def test_html_is_lowercased():
     )
 
 
-def test_invalid_uri_does_not_raise_error():
-    assert clean('<a href="http://example.com]">text</a>') == "<a>text</a>"
-
-
 @pytest.mark.parametrize(
     "data, should_strip, expected",
     [
@@ -143,41 +159,47 @@ def test_bare_entities_get_escaped_correctly(text, expected):
 @pytest.mark.parametrize(
     "text, expected",
     [
-        # Test character entities
+        # Test character entities in text don't get escaped
         ("&amp;", "&amp;"),
         ("&nbsp;", "&nbsp;"),
         ("&nbsp; test string &nbsp;", "&nbsp; test string &nbsp;"),
         ("&lt;em&gt;strong&lt;/em&gt;", "&lt;em&gt;strong&lt;/em&gt;"),
-        # Test character entity at beginning of string
+        # Test character entity at beginning of string doesn't get escaped
         ("&amp;is cool", "&amp;is cool"),
-        # Test it at the end of the string
+        # Test character entity at end of the string doesn't get escaped
         ("cool &amp;", "cool &amp;"),
-        # Test bare ampersands and entities at beginning
+        # Test bare ampersands before an entity at the beginning of the string
+        # gets escaped
         ("&&amp; is cool", "&amp;&amp; is cool"),
-        # Test entities and bare ampersand at end
+        # Test ampersand after an entity at the end of the string gets escaped
         ("&amp; is cool &amp;&", "&amp; is cool &amp;&amp;"),
-        # Test missing semi-colon means we don't treat it like an entity
+        # Test missing semi-colons mean we don't treat the thing as an entity--Bleach
+        # only recognizes character entities that start with & and end with ;
         ("this &amp that", "this &amp;amp that"),
-        # Test a thing that looks like a character entity, but isn't because it's
-        # missing a ; (&current)
         (
             "http://example.com?active=true&current=true",
             "http://example.com?active=true&amp;current=true",
         ),
-        # Test character entities in attribute values are left alone
+        # Test character entities in attribute values are not escaped
         ('<a href="?art&amp;copy">foo</a>', '<a href="?art&amp;copy">foo</a>'),
         ('<a href="?this=&gt;that">foo</a>', '<a href="?this=&gt;that">foo</a>'),
-        # Ambiguous ampersands get escaped in attributes
+        # Things in attributes that aren't character entities get escaped
         (
             '<a href="http://example.com/&xx;">foo</a>',
             '<a href="http://example.com/&amp;xx;">foo</a>',
         ),
+        (
+            '<a href="http://example.com?&adp;">foo</a>',
+            '<a href="http://example.com?&amp;adp;">foo</a>',
+        ),
         (
             '<a href="http://example.com?active=true&current=true">foo</a>',
             '<a href="http://example.com?active=true&amp;current=true">foo</a>',
         ),
-        # Ambiguous ampersands in text are not escaped
-        ("&xx;", "&xx;"),
+        # Things in text that aren't character entities get escaped
+        ("&xx;", "&amp;xx;"),
+        ("&adp;", "&amp;adp;"),
+        ("&currdupe;", "&amp;currdupe;"),
         # Test numeric entities
         ("&#39;", "&#39;"),
         ("&#34;", "&#34;"),
@@ -302,20 +324,6 @@ def test_stripping_tags_is_safe(data, expected):
     assert clean(data, strip=True) == expected
 
 
-def test_allowed_styles():
-    """Test allowed styles"""
-    ATTRS = ["style"]
-    STYLE = ["color"]
-
-    assert clean('<b style="top:0"></b>', attributes=ATTRS) == '<b style=""></b>'
-
-    text = '<b style="color: blue;"></b>'
-    assert clean(text, attributes=ATTRS, styles=STYLE) == text
-
-    text = '<b style="top: 0; color: blue;"></b>'
-    assert clean(text, attributes=ATTRS, styles=STYLE) == '<b style="color: blue;"></b>'
-
-
 def test_href_with_wrong_tag():
     assert clean('<em href="fail">no link</em>') == "<em>no link</em>"
 
@@ -469,10 +477,31 @@ def test_attributes_list():
 @pytest.mark.parametrize(
     "data, kwargs, expected",
     [
+        # invalid URI (urlparse raises a ValueError: Invalid IPv6 URL)
+        # is not allowed by default
+        (
+            '<a href="http://example.com]">text</a>',
+            {"protocols": ALLOWED_PROTOCOLS},
+            "<a>text</a>",
+        ),
+        # data protocol is not allowed by default
+        (
+            '<a href="data:text/javascript,prompt(1)">foo</a>',
+            {"protocols": ALLOWED_PROTOCOLS},
+            "<a>foo</a>",
+        ),
         # javascript: is not allowed by default
-        ("<a href=\"javascript:alert('XSS')\">xss</a>", {}, "<a>xss</a>"),
+        (
+            "<a href=\"javascript:alert('XSS')\">xss</a>",
+            {"protocols": ALLOWED_PROTOCOLS},
+            "<a>xss</a>",
+        ),
         # File protocol is not allowed by default
-        ('<a href="file:///tmp/foo">foo</a>', {}, "<a>foo</a>"),
+        (
+            '<a href="file:///tmp/foo">foo</a>',
+            {"protocols": ALLOWED_PROTOCOLS},
+            "<a>foo</a>",
+        ),
         # Specified protocols are allowed
         (
             '<a href="myprotocol://more_text">allowed href</a>',
@@ -486,12 +515,23 @@ def test_attributes_list():
             "<a>invalid href</a>",
         ),
         # Anchors are ok
+        (
+            '<a href="#section-1">foo</a>',
+            {"protocols": []},
+            '<a href="#section-1">foo</a>',
+        ),
+        # Anchor that looks like a domain is ok
         (
             '<a href="#example.com">foo</a>',
             {"protocols": []},
             '<a href="#example.com">foo</a>',
         ),
         # Allow implicit http if allowed
+        (
+            '<a href="/path">valid</a>',
+            {"protocols": ["http"]},
+            '<a href="/path">valid</a>',
+        ),
         (
             '<a href="example.com">valid</a>',
             {"protocols": ["http"]},
@@ -522,6 +562,14 @@ def test_attributes_list():
             {"protocols": ["http"]},
             '<a href="192.168.100.100:8000">valid</a>',
         ),
+        pytest.param(
+            *(
+                '<a href="192.168.100.100:8000/foo#bar">valid</a>',
+                {"protocols": ["http"]},
+                '<a href="192.168.100.100:8000/foo#bar">valid</a>',
+            ),
+            marks=pytest.mark.xfail,
+        ),
         # Disallow implicit http if disallowed
         ('<a href="example.com">foo</a>', {"protocols": []}, "<a>foo</a>"),
         ('<a href="example.com:8000">foo</a>', {"protocols": []}, "<a>foo</a>"),
@@ -572,7 +620,7 @@ def test_svg_attr_val_allows_ref():
     [
         (
             '<svg><pattern id="patt1" href="#patt2"></pattern></svg>',
-            '<svg><pattern href="#patt2" id="patt1"></pattern></svg>',
+            '<svg><pattern id="patt1" href="#patt2"></pattern></svg>',
         ),
         (
             '<svg><pattern id="patt1" xlink:href="#patt2"></pattern></svg>',
@@ -1022,6 +1070,42 @@ def test_html_comments_escaped(namespace_tag, end_tag, eject_tag, data, expected
     )
 
 
+@pytest.mark.parametrize(
+    "text, expected",
+    [
+        (
+            "<p>Te<b>st</b>!</p><p>Hello</p>",
+            "Test!\nHello",
+        ),
+        (
+            # with an internal space and escaped character
+            "<p>This is our <b>description!</b> &amp;</p><p>nice!</p>",
+            "This is our description! &amp;\nnice!",
+        ),
+        (
+            # note: double-wrap causes an initial newline--this can't really be
+            # handled under the current design
+            "<div><p>This is our <b>description!</b> &amp;</p></div><p>nice!</p>",
+            "\nThis is our description! &amp;\nnice!",
+        ),
+        (
+            # newlines are used to keep lists and other elements readable
+            (
+                "<div><p>This is our <b>description!</b> &amp;</p><p>1</p>"
+                "<ul><li>a</li><li>b</li><li>c</li></ul></div><p>nice!</p>"
+            ),
+            "\nThis is our description! &amp;\n1\n\na\nb\nc\nnice!",
+        ),
+    ],
+)
+def test_strip_respects_block_level_elements(text, expected):
+    """
+    Insert a newline between block level elements
+    https://github.com/mozilla/bleach/issues/369
+    """
+    assert clean(text, tags=[], strip=True) == expected
+
+
 def get_ids_and_tests():
     """Retrieves regression tests from data/ directory
 
@@ -1063,6 +1147,13 @@ def test_regressions(test_case):
     assert clean(test_data) == expected
 
 
+def test_preserves_attributes_order():
+    html = """<a target="_blank" href="https://example.com">Link</a>"""
+    cleaned_html = clean(html, tags=["a"], attributes={"a": ["href", "target"]})
+
+    assert cleaned_html == html
+
+
 class TestCleaner:
     def test_basics(self):
         TAGS = ["span", "br"]
@@ -1092,4 +1183,4 @@ class TestCleaner:
         cleaner = Cleaner(tags=TAGS, attributes=ATTRS, filters=[MooFilter])
 
         dirty = 'this is cute! <img src="http://example.com/puppy.jpg" rel="nofollow">'
-        assert cleaner.clean(dirty) == 'this is cute! <img rel="moo" src="moo">'
+        assert cleaner.clean(dirty) == 'this is cute! <img src="moo" rel="moo">'
